<template>
  <div>
    <el-upload
      class="upload-demo"
      :action="defaultSettings.uploadUrl"
      :multiple="defaultSettings.multiple"
      name="multipartFile"
      :accept="defaultSettings.accept"
      :file-list="showList"
      :list-type="defaultSettings.type"
      :limit="defaultSettings.num"
      :disabled="defaultSettings.disabled"
      :on-exceed="handleExceed"
      :on-preview="handlePreview"
      :on-remove="handleRemove"
      :before-upload="beforeUpload"
      :on-success="handleUploadSuccess"
      :headers="headers"
      :drag="settings.drag"
    >
      <div v-if="settings.type==='picture-card'">
        <template>
          <i class="el-icon-plus"></i>
        </template>
      </div>
      <div v-else>
        <template v-if="settings.drag">
          <i class="el-icon-upload" />
          <div class="el-upload__text">将文件拖到此处，或<em>点击上传</em></div>
        </template>
        <el-button v-if="!settings.drag" size="medium" type="text" plain icon="el-icon-upload">
          {{ defaultSettings.title }}
        </el-button>
      </div>
      <div v-if="defaultSettings.tip" slot="tip" class="el-upload__tip">{{ defaultSettings.tip }}</div>
      <div v-else slot="tip" class="el-upload__tip">只能上传{{ defaultSettings.num }}个格式为[{{ defaultSettings.accept }}]的文件，且不超过{{ defaultSettings.max }}M
      </div>
    </el-upload>
    <div id="uploadPreviewImages" style="display:none;">
      <span
        v-for="(src,index) in fileList"
        :key="index"
      >
        <img
          v-if="checkImg(src.name)"
          class="v-img"
          :src="getUrl(src.path)"
          :alt="src.name"
          style="width: 100px;height: 100px;"
        >
      </span>
    </div>
  </div>
</template>

<script>
import {getToken} from '@/utils/auth'
import {sessionToken} from '@/settings'
import {getDownUrl} from '@/utils/base'
import Viewer from 'viewerjs'
import 'viewerjs/dist/viewer.css'

let viewer = null
export default {
  props: {
    values: {
      type: Array,
      default: function() {
        return []
      }
    },
    settings: {
      type: Object,
      default: function() {
        return {}
      }
    }
  },
  data() {
    return {
      headers: {},
      showList: [], // 用于详情展示
      fileList: [],
      defaultSettings: {
        title: '附件上传',
        max: 100, // 最大大小，单位M
        num: 10, // 支持上传图片个数
        accept: '.jpg,.jpeg,.png,.pdf,.doc,.docx,.xls,.xlsx', // 限制格式
        tip: '', // 提示 默认：`只能上传${this.defaultSettings.num}个${this.defaultSettings.accept}文件，且不超过${this.defaultSettings.max}kb`
        uploadUrl: '', // 上传路径
        multiple: true, // 是否支持批量上传
        disabled: false, // 是否禁用
        type: 'text' // text/picture
      }
    }
  },
  watch: {
    values(v) {
      this.fileList = Object.assign([], v)
      this.showList = Object.assign([], v)
      this.$nextTick(() => {
        this.initPreviewImg()
      })
    }
  },
  created() {
    this.showList = Object.assign([], this.values)
    this.fileList = Object.assign([], this.values)

    this.showList.forEach(obj => {
      obj.url = this.getUrl(obj.path)
    })

    this.fileList.forEach(obj => {
      obj.url = this.getUrl(obj.path)
    })

    this.defaultSettings = Object.assign(this.defaultSettings, this.settings)
    //
    // this.$nextTick(() => {
    //   new Viewer(document.getElementsByClassName('el-upload-list'))
    // })
  },
  mounted() {
    this.initPreviewImg()
    this.headers[sessionToken] = getToken()
  },
  methods: {
    handleRemove(file, fileList) {
      for (let i = 0; i < this.fileList.length; i++) {
        const img = this.fileList[i]
        if (file.name == img.name) {
          this.fileList.splice(i, 1)
          break
        }
      }
      this.$nextTick(() => {
        this.$emit('on-change')
        this.initPreviewImg()
      })
    },
    handleExceed(files, fileList) {
      this.$message.warning('最多上传' + this.defaultSettings.num + '个文件')
    },
    beforeUpload(file) {
      let isRepeat = true
      for (let i = 0; i < this.fileList.length; i++) {
        const f = this.fileList[i]
        if (file.name == f.name) {
          isRepeat = false
          break
        }
      }
      if (!isRepeat) {
        this.$message.error('该文件已存在!')
      }
      const isImg = this.defaultSettings.accept.split(',').indexOf(file.name.substr(file.name.lastIndexOf('.'), file.name.length).toLocaleString()) > -1
      const isLt2M = file.size / 1024 / 1024 < Number(this.defaultSettings.max)
      if (!isImg) {
        this.$message.error('上传文件格式只能是 ' + this.defaultSettings.accept + ' 格式!')
      }
      if (!isLt2M) {
        this.$message.error('上传文件大小不能超过 ' + this.defaultSettings.max + 'M!' + '当前文件大小：' + (file.size / 1024 / 1024).toFixed(2) + 'M')
      }
      return isImg && isLt2M && isRepeat
    },
    handleUploadSuccess(response, file, fileList) {
      if (response.code === 1) {
        response.data.forEach(f => {
          this.fileList.push({
            fileType:f.fileExt,
            id: f.id,
            name: f.fileName,
            url: this.getUrl(f.url),
            path: f.url,
            attSize: f.fileSize || null
          })
        })
        this.$nextTick(() => {
          this.$emit('on-change')
          this.initPreviewImg()
        })
      } else {
        this.$message.warning('上传失败')
      }
    },
    getContent() {
      return this.fileList
    },
    handlePreview(file) {
      if (!this.checkImg(file.name)) {
        return false
      }
      let index = 0
      for (let i = 0; i < this.fileList.length; i++) {
        const f = this.fileList[i]
        if (this.checkImg(f.name)) {
          if (file.id == f.id) {
            break
          }
          index++
        }
      }
      // this.fileList.forEach((f, i) => {
      //   if (file.uid == f.uid) {
      //     index = i
      //   }
      // })
      // document.querySelector('#uploadPreviewImages').children[0].click()
      viewer.view(index)
    },
    /*
     * @Author : liu.q [916000612@qq.com]
     * @Date : 2019-05-26 14:48
     * @Description : 初始化
     */
    initPreviewImg() {
      if (viewer != null) {
        viewer.destroy()
      }
      const ViewerDom = document.querySelector('#uploadPreviewImages')
      viewer = new Viewer(ViewerDom, {
      })
      //
    },
    /*
     * @Author : liu.q [916000612@qq.com]
     * @Date : 2019-05-26 14:48
     * @Description : 校验图片
     */
    checkImg(name) {
      const suffix = name.substring(name.lastIndexOf('.'), name.length)
      const imgArray = ['.jpg', '.jpeg', '.png', '.bmp']
      if (imgArray.indexOf(suffix) < 0) {
        return false
      }
      return true
    },
    getUrl(path) {
      if (path.substr(0, 7).toLowerCase() == 'http://' || path.substr(0, 8).toLowerCase() == 'https://') {
        return path
      } else {
        return getDownUrl() + path
      }
    }
  }
}
</script>

<style scoped>
  /* 图片预览层级*/
  .viewer-container {
    z-index: 90709102!important;
  }

</style>
